library ion;

import 'dart:io';
import 'src/ion_sock.dart';
import 'src/ion_req.dart';
import 'src/ion_res.dart';

class Ion {
  int port;
  InternetAddress ip;
  HttpServer srv;

  IonWebsocketController ws_ctl;

  Map ws_cfg = {
    "ws_cnx": null,
    "ws_wlc": null,
  };

  final paths = {
    "GET": {},
    "POST": {},
    "PUT": {},
    "DELETE": {}
  };

  final cfg;
  final production;

  Ion(this.port, {Map this.cfg, bool this.production = false}) {
    _init();
  }

  _init() async {
    ip = InternetAddress.anyIPv4;

    srv = await HttpServer.bind(ip, port);

    await for (HttpRequest req in srv) {
      var res = req.response;
      var ion_res = IonResponse(res);
      var ion_req = IonRequest(req);
      
      await ion_req.init();

      var path = ion_req.url.split("?")[0];

      if (!_handle(path, ion_req, ion_res)) {
        var file = File(path);

        if (file.existsSync()) {
          ion_res.send(file);
        } 
        else if (path == "/ion-ws-connect/" + ws_cfg["ws_cnx"]) {
          var ws = await WebSocketTransformer.upgrade(req);
          ws_ctl.transform(ws);
          if (ws_cfg["ws_wlc"] is Function) {
            ws_cfg["ws_wlc"](ws);
          }
        } else {
          print("Cannot ${req.method}: $path");
          ion_res.send("Cannot ${req.method}: $path");
        }
      }

      res.close();
    }
  }

  _handle(path, req, res) {
    var isHandled = false;
    paths[req.method].forEach((key, value) {
      if (key == path) {
        value(req, res);
        isHandled = true;
        return true;
      } else if (key.contains(":")) {
        var params = {};
        var segments = key.split("/");
        var param_indicies = segments.map((el) => el.startsWith(":")).toList();

        var path_segments = path.split("/");

        var isValid = true;

        for (int i = 0; i < param_indicies.length; i++) {
          bool isParam = param_indicies[i];

          if (!isParam) {
            if (segments[i] != path_segments[i]) {
              isValid = false;
              break;
            }
          } else {
            String sg = segments[i];
            params[sg.replaceFirst(r":", "")] = path_segments[i];
          }
        }
        if (isValid) {
          req.setParams(params);
          value(req, res);

          isHandled = true;
          return true;
        }
      }
    });

    return isHandled;
  }

  get(uri, fn) {
    paths["GET"][uri] = fn;
  }

  post(uri, fn) {
    paths["POST"][uri] = fn;
  }

  put(uri, fn) {
    paths["PUT"][uri] = fn;
  }

  delete(uri, fn) {
    paths["DELETE"][uri] = fn;
  }
  
  all(uri, fn) {
    paths[uri] = fn;
  }

  ws([String url, Function fn]) {
    if (ws_ctl == null) {
      ws_ctl = IonWebsocketController();
    }

    if (url is String) {
      ws_cfg["ws_cnx"] = url;
      if (fn is Function) {
        ws_cfg["ws_wlc"] = fn;
      }
    }
    return ws_ctl;
  }
}
