import "dart:io";
import "dart:convert";

class IonWebsocketController {
  var _sockets = <IonWebsocket>[];

  Function _connect = () {};

  send(title, data) {
    _sockets.forEach((ws) {
      ws.send(title, data);
    });
  }

  emit(title, data) {
    send(title, data);
  }

  _broadcast(client, title, data) {
    if (data is Map) {
      data = json.encode(data);
    }
    _sockets.forEach((ws) {
      if(ws != client){
        ws.send(title, data);
      }
    });
  }

  connect(Function fn) {
    _connect = fn;
  }

  transform(WebSocket ws) {
    var i = IonWebsocket(ws, this);
    _sockets.add(i);

    _connect(i);
  }
}

class IonWebsocket {
  final WebSocket o;
  final ctl;

  var behaviours = {};

  Function _disconnect = () {};

  IonWebsocket(this.o, this.ctl) {
    var str = o.listen(_onMessage());
    str.onDone(_onDisconnect());
  }

  send(title, data) {
    var payload = json.encode([title, data]);
    o.add(payload);
  }

  emit(title, data) {
    send(title, data);
  }

  broadcast(title, data) {
    ctl._broadcast(this, title, data);
  }

  disconnect(fn) {
    _disconnect = fn;
  }

  on(String handler, Function fn) {
    behaviours[handler] = fn;
  }

  Function _onMessage() {
    return (message) {
      var msg = json.decode(message);

      var title = msg[0];
      var content = msg[1];

      behaviours.forEach((behaviour, function) {
        if (behaviour == title) {
          function(content);
          return;
        }
      });
    };
  }

  Function _onDisconnect() {
    return () {
      this._disconnect();
      ctl._sockets.remove(this);
    };
  }
}
